!> \file icetemp_declar-0.5_mod.f90
!! Declar local variables for the temperature calculation in ice 
!<


!> \namespace icetemp_declar 
!! module to declar local variables for the temperature calculation in ice 
!! @note used modules:
!! @note - Module_choix
!! @note - Param_phy_mod
!<
module icetemp_declar     

  Use Module_choix
  Use Param_phy_mod

  Implicit None 

  !> \Var
!  integer ::  Nx_m,Ny_m,Nz_m,Nzm_m,NzNzm_m 
  Integer :: Iec,Jec,Ngr
  Integer :: Nfracq   !< Exposant Fracq
  Integer :: Iq       !< Choix Du Type De Routine Chaleur
  Real :: Sx,Sy,Sx2,Sy2,Deh22,Tss
  Real ::  Dou,Dah,Duu,Dzz,Dzi,Chalbed,Ct_bas,Ct_haut

  Real,Parameter :: Acof1=-0.0575  !< Pour La Temperature De L'Eau De Mer
  Real,Parameter :: Bcof1=0.0901   !< Pour La Temperature De L'Eau De Mer
  Real,Parameter :: Ccof1=7.61e-4  !< Pour La Temperature De L'Eau De Mer
  Real,Parameter :: S0mer=34.75    !< Pour La Temperature De L'Eau De Mer

  ! a la Base De L'Ice Shelf
  Integer :: Ifail1             !< Permet De Detecter Les Erreurs 
  Real :: Chalk_1               !< Utilise Pour Le Calcul De Chalk : Glace Posée
  Real :: Chalk_2               !< Utilise Pour Le Calcul De Chalk : Ice Streams Et Ice Shelves
  Real :: Chalbed_1             !< Utilise Pour Le Calcul De Chalbed
  Real :: Coefadv               !< Pour Limiter Le Flux De Chaleur Horiz.
  Real :: Ecart_phid            !< Pour Limiter L'Impact De La Chaleur De Glissement Si Base Froide
  ! Ancienne Valeur 0.5.

  ! Tableaux 1d
  ! _______________

  Real,allocatable,Dimension(:) :: Aa    !< Work Arrays For Tridiag  !Dim Nn
  Real,allocatable,Dimension(:) :: Bb    !< Work Arrays For Tridiag  !Dim Nn
  Real,allocatable,Dimension(:) :: Cc    !< Work Arrays For Tridiag  !Dim Nn
  Real,allocatable,Dimension(:) :: Rr    !< Work Arrays For Tridiag  !Dim Nn
  Real,allocatable,Dimension(:) :: Hh    !< Work Arrays For Tridiag  !Dim Nn

  Real,allocatable,Dimension(:) :: Tdot   !< Temperature Time Derivative*Dtt  
  Real,allocatable,Dimension(:) :: Abis,Bbis,Cbis,Rbis,Hbis                   
  Real,allocatable,Dimension(:) :: Ee        !< Vertical Coordinate In Ice, Scaled To H Zeta  

  ! Tableaux De Travail 2d 
  ! ___________________________
  Real,allocatable,Dimension(:,:):: Tbmer    !< Temperature De La Mer A La Base De L'Ice Shelf  
  Real,allocatable,Dimension(:,:) :: Chalglissx,Chalglissy !< Chaleur De Glissement             
  Integer,allocatable,Dimension(:,:) :: Iadvec_w,Iadvec_e,Iadvec_s,Iadvec_n   
  ! Pour L'Advection        

  ! Tableaux De Travail 3d
  ! __________________________
  Real,allocatable,Dimension(:,:,:,:) :: Chalx, Chaly         !Dim Nx,Ny,Nz,N1poly:N2poly
  ! Utilise Pour Calculer La Chaleur De Deformation Selon Xx Yy Zz Et Xy
  Real,allocatable,Dimension(:,:,:) :: Ffx,Ffy                                       
  Real,allocatable,Dimension(:,:,:) :: T3d_new                      
  Real,allocatable,Dimension(:,:,:) :: Chal2_x, Chal2_y, Chal2_z, Chal2_xy
  Real,allocatable,Dimension(:,:,:) :: Chaldef_maj   !< Chaleur De Deformation                   
  Real,allocatable,Dimension(:,:,:) :: Advecx,Advecy,Advec
  Real,Dimension(Nx,Ny,Nz) :: Cp            !< Specific Heat Capacity (J/(M-3)/K)=Ro Cp  
  Real,Dimension(Nx,Ny,Nz) :: Ct            !< Thermal Conductivity (J/M/K/A)            


  !Variables De Travail
  Real ::  Fracq                   !< For Temp Calculation = (1.-(1.-De/2.)**5)/5.
  Integer,Parameter :: Ncond=0     !< Switch : Ncond=1 Thermal Conductivity In Mantle
  Real :: Dee                      !< Vertical Step In Ice And Mantle
  Real,Parameter :: Dzm=600        !< Grid Step In Mantle
  Real :: Dttdx                    !< For Temp Calculation = Dtt/Dx
  Real :: Dx11                     !< 1/Dx

  !Prop Thermique 
  Real,Parameter ::  Cm=1.04e8              !< Thermal Conductivity Of Mantle J/M/K/A
  Real,Parameter ::  Cpm=1000.0             !< Specific Heat Capacity Of Mantle J/Kg/K
  Real:: Ctm                                !< For Temp Calculation


end module icetemp_declar

