!> \file icetemp_mod.f90
!! Interfaces of functions and subroutines for temperature calculation 
!<

!> \namespace icetempmod
!! Interfaces of functions and subroutines for the temperature calculation 
!<

module icetempmod

contains

  !> SUBROUTINE: init_icetemp 
  !! Initialise the temperature calculation
  !! Used modules:
  !! - Icetemp_declar
  !>

  Subroutine init_icetemp(Num_rep_42_m)
    Use Icetemp_declar, only : Nfracq,Iq,Ecart_phid

    Implicit None

    !< Arguments
    Integer,                       Intent(In)::Num_rep_42_m !< Id Of Reponse
    ! Character(Len=80) :: Filin
    !________________________________________________________________
    ! La Temperature Du Point De Congelation De L'Eau De Mer
    ! Est Tiree De Jenkins (1991), Tb Est En Degres C
    ! _______________________________________________________________

    !  Nouvelle Formulation De Fracq
    !  Nfracq=15  ! Ancienne Formulation

    Nfracq=1

    Write(Num_rep_42_m,*) 'Calcul Des Temperatures'
    Write(Num_rep_42_m,*) 'Nfracq=',Nfracq

    Iq=1
    If (Iq.Eq.1) Write(Num_rep_42_m,*) 'Chaleur Demi Maille, Iq=',Iq
    If (Iq.Eq.2) Write(Num_rep_42_m,*) 'Chaleur Centree, Iq=',Iq
    If (Iq.Eq.3) Write(Num_rep_42_m,*) 'Chaleur Pente, Iq=',Iq
    If (Iq.Eq.4) Write(Num_rep_42_m,*) 'Chaleur U_taub, Iq=',Iq
    If (Iq.Eq.5) Write(Num_rep_42_m,*) 'Chaleur U_taub_stag, Iq=',Iq
    If (Iq.Eq.6) Write(Num_rep_42_m,*) 'Chaleur Q_sia_stag,Iq=',Iq
    If (Iq.Eq.7) Write(Num_rep_42_m,*) 'Chaleur Q_all_stag,Iq=',Iq
    Write(Num_rep_42_m,*)

    Ecart_phid=0.5
    Write(Num_rep_42_m,*) ' Si Base Froide, La Chaleur De Glissement Est En Exp(Delta T * Ecart_phid)'
    Write(Num_rep_42_m,*) 'Ecart_phid=',Ecart_phid
    Write(Num_rep_42_m,*) '---------------------------------------------------------------------------'

  End Subroutine init_icetemp
  

  !-----------------------------------------------------------------------------------------------------------------------

  !> SUBROUTINE: icetemp
  !! Calculate the temperature
  !!
  !! Used modules:
  !!  -  use Icetemp_declar 
  !>

  subroutine icetemp

    use Icetemp_declar


    Implicit None 

    ! Allocation de la memoire pour les tableaux dans icetemp_declar

    If (.Not. Allocated(Aa)) then 
       ! tab Nn
       Allocate (Aa(Nn),Bb(Nn),Cc(Nn),Rr(Nn),Hh(Nn))  
       ! tab NzNzm_m
       Allocate(Tdot(Nz+Nzm))   
       ! tab Nzm+1
       Allocate(Abis(Nzm+1),Bbis(Nzm+1),Cbis(Nzm+1),Rbis(Nzm+1),Hbis(Nzm+1))                  
       ! tab nz_m
       Allocate(Ee(Nz))        
       !Tab Nx,Ny
       Allocate(Tbmer(Nx,Ny),&   
            Chalglissx(Nx,Ny),&
            Chalglissy(Nx,Ny),&
            Iadvec_w(Nx,Ny),&
            Iadvec_e(Nx,Ny),&
            Iadvec_s(Nx,Ny),&
            Iadvec_n(Nx,Ny) )      
       ! Tab Nx,Ny,Nz
       Allocate(Chal2_x(Nx,Ny,Nz),&
            Chal2_y(Nx,Ny,Nz),&
            Chal2_z(Nx,Ny,Nz),&
            Chal2_xy(Nx,Ny,Nz),&
            Chaldef_maj(Nx,Ny,Nz),&
            Advecx(Nx,Ny,Nz),&
            Advecy(Nx,Ny,Nz),&
            Advec(Nx,Ny,Nz))
       ! Tab Nx,Ny,Nz,N1poly:N2poly
       Allocate(Chalx(Nx,Ny,Nz,size(Btt,4)),&
            Chaly(Nx,Ny,Nz,size(Btt,4)))         
       ! Tab Nx,Ny,N1poly:N2poly
       Allocate(Ffx( Nx,Ny,size(Btt,4)),&
            Ffy( Nx,Ny,size(Btt,4)))                                        
       ! Tab Nx,Ny,NzNzm
       Allocate( T3d_new (Nx,Ny,Nz+Nzm))                  
    end If

    ! init
    Aa=0.    
    Bb=0.              
    Cc=1.             
    Rr=0.              
    Hh=0.              
    Tdot=0
    Chal2_x=0.
    Chal2_y=0.
    Chal2_z=0.
    Chal2_xy=0.
    Chaldef_maj=0.
    Chalglissx=0.
    Chalglissy=0.
    !Instruction 
    Dee=1./(Nz-1) 
    Fracq=(1.-(1.-Dee/2.)**Nfracq)/Nfracq
    Fracq=(1.-(1.-Dee/2.)**5.)/5.
    Ee(1)=0.
    Ee(Nz)=1.
    Do K=1,Nz
       If ((K.Ne.1).And.(K.Ne.Nz)) Ee(K)=(K-1.)/(Nz-1.)
    End Do

    !   Variables Dependant Du Pas De Temps Dtt
    Ctm=Dtt*Cm/Rom/Cpm/Dzm/Dzm
    Dttdx=Dtt/Dx
    Dx11=1./Dx

    If (Itracebug.Eq.1) Write(num_tracebug,*)' Entree Dans Routine Ice_temp Time=',Time

    !________________________________________________________________
    ! La Temperature Du Point De Congelation De L'Eau De Mer
    ! Est Tiree De Jenkins (1991), Tb Est En Degres C
    ! _______________________________________________________________
    If (Itracebug.Eq.1) Write(num_tracebug,*)' avant appel  Routine Temp_mer'
    Call Temp_mer

    ! Appel Aux Proprietes Thermiques De La Glace : Conductivite, Capacite Claorifique -Tpmp

    If (Itracebug.Eq.1) Write(num_tracebug,*)' avant appel  Routine Thermal_property'
    Call Thermal_prop_icetemp

    If (Itracebug.Eq.1) Write(num_tracebug,*)' avant appel  Routine Qprod'
    Call Qprod_ice(Iq)


    ! Dans Le Socle : Les elements de La Matrice Tridiag Sont Invariants Dans L'Espace
    Do K=Nz+1,nz+nzm-1
       Aa(K)=-Ctm
       Bb(K)=1.+2.*Ctm
       Cc(K)=-Ctm
    End Do

    ! Conditions Aux Limites Pour Le Socle
    Aa(nz+nzm)=-1.
    Bb(nz+nzm)=1.
    Cc(nz+nzm)=0.

    ! Calcul D'Advection : On Traite Les Tableaux En Une Seule Ligne De Commandes
    ! les tableaux iadvec valent 1 quand la vitesse consideree est upwind et 0 sinon

    !         Advection Selon X
    !         -----------------

    Iadvec_w(:,:)=Nint(Max(Sign(1.0,Uxbar(:,:)),0.0))
    Iadvec_e(:,:)=Nint(Abs(Min(Sign(1.0,Uxbar(:,:)),0.0)))

    !         Advection Selon Y
    !         -----------------

    Iadvec_s(:,:)=Nint(Max(Sign(1.0,Uybar(:,:)),0.0)) 
    Iadvec_n(:,:)=Nint(Abs(Min(Sign(1.0,Uybar(:,:)),0.0)))

    ! Boucle De Resolution Des Temperatures Colonne Par Colonne

    If (Itracebug.Eq.1) Write(num_tracebug,*)' avant appel  boucle Advec_icetemp'

    Do J=2,Ny-1
       Do I=2,Nx-1

 !  If (Itracebug.Eq.1) Write(num_tracebug,*)' avant appel  Advect_icetemp  i,j', i,j
          call Advec_icetemp(I,J,Chaldef_maj(I,J,:),Cp(I,J,:),& 
               Advecx(I,J,:),Advecy(I,J,:),Advec(I,J,:),Ct(I,J,:))

          ! variables de calcul dans la glace
          Deh22=2.*Dee*H(I,J)**2
          Duu=Dtt/2./Dee
          Rr(nz+nzm)=-Dzm*Ghf(I,J)/Cm  ! depend de I,J Pour Un Flux Geothermique Variable

!   If (Itracebug.Eq.1) Write(num_tracebug,*)' avant appel  temp_col  i,j', i,j
          Call temp_col(I,J,T3d_new,Ct(I,J,:),&
               flot(I,J),H(I,J),Tbmer(I,J))

          ! Test Permettant D'Ecrire Les Variables Quand Il Y A Une Erreur
          ! Ifail1 = 1 : Erreur Dans Tridiag
          If (Ifail1.Eq.1) Then
             Write(6,*)'Erreur Dans Tridiag'
             Write(6,*)'I = ',I,' J = ',J
             Write(6,*) 'A, B, C, R, U'
             Do K=1,nz+nzm
                Write(6,*)'K = ',K
                !Write(6,*) Aa(K),Bb(K),Cc(K),Rr(K),Hh(K)
             End Do
             Stop
          End If
       End Do
    End Do

    Do J=1,Ny
       Do I=1,Nx
          H1(I,J)=H(I,J)
          B1(I,J)=B(I,J)
          Tpmp(I,J,1)=0.
       End Do
    End Do

    ! Attribution Finale De La Temperature
    ! Limitation a 3 deg De Variation Par Dtt Et Pas Plus Froid Que -70

    Do K=1,nz+nzm
       Do J=1,Ny
          Do I=1,Nx
             If (H(I,J).Gt.10.) Then

                Tdot(K)=T3d_new(I,J,K)-T(I,J,K)

                If ((K.Gt.1).And.(Tdot(K).Lt.-3.)) Tdot(K)=-3.
                If ((K.Gt.1).And.(Tdot(K).Gt.3.)) Tdot(K)=3.

                T(I,J,K)=T(I,J,K)+Tdot(K)

                If (T(I,J,K).Lt.-70.) Then
                   T(I,J,K)=-70.
                Endif
             Endif

          End Do
       End Do
    End Do


    Do K=2,Nz
       Do J=1,Ny
          Do I=1,Nx
             If (T(I,J,K).Gt.Tpmp(I,J,K)) Then
                T(I,J,K)=Tpmp(I,J,K)
                Ibase(I,J)=2
             Endif
          End Do
       End Do
    End Do

    !     Temperature Sur Les Bords :
    !     -------------------------
  
    Do K=1,Nz
       Do J=1,Ny
          !        T(Nx,J,K)= T(Nx-1,J,K)
          !        T(1,J,K)= T(2,J,K) 
          T(Nx,J,K)= Ts(Nx,J)
          T(1,J,K)= Ts(1,J)
       End Do
       !
       Do I=1,Nx
          !        T(I,1,K)=T(I,2,K) 
          !        T(I,Ny,K)=T(I,Ny-1,K) 
          T(I,1,K)=Ts(I,1)
          T(I,Ny,K)=Ts(I,Ny)
       End Do
    End Do
   

    Do K=1,Nz
       Do J=1,Ny
          Do I=1,Nx
             If (H(I,J).le.1.) then 
                T(I,J,K)=Ts(I,J)
                if ((k.ge.nz).and.(flot(i,j))) T(I,J,K) =Tbmer(I,J)
             end If
                
          End Do
       End Do
    End Do

    Deallocate (Aa,Bb,Cc,Rr,Hh,Tdot,Abis,Bbis,Cbis,Rbis,Hbis,Ee,Tbmer,            & 
                Chalglissx,Chalglissy,Iadvec_w,Iadvec_e,                          & 
                Iadvec_s,Iadvec_n,Chal2_x,Chal2_y,Chal2_z,Chal2_xy,Chaldef_maj,   &
                Advecx,Advecy,Advec,Chalx,Chaly,Ffx,Ffy,T3d_new)        

    If (Itracebug.Eq.1) Write(num_tracebug,*)' fin routine icetemp'
    return
  End Subroutine icetemp

  !-----------------------------------------------------------------------------------------------------------------------

end module icetempmod
