!> \file relaxation_water_mod-0.4.f90
!! Module pour le calcule de l'ecoulement de l'eau sous la glace  
!! selon une equation de Darcy avec un shema amont.     
!<

!> \namespace RELAXATION_WATER_MOD
!! Module pour le calcule de l'ecoulement de l'eau sous la glace  
!! selon une equation de Darcy avec un shema amont.    
!! \author Vincent Peyaud 
!! \date septembre 2004
!! @note  reprise de relaxation_mod (pour le calcul de l'epaisseur)            
!!     et adaptation pour le calcul de l'ecoulement de l'eau sous la glace  
!!     selon une equation de Darcy avec un shema amont.                     
!<


! ************************************************************************ !
!                                                                          !                                      
!                       Vincent Peyaud septembre 2004                      !
!     reprise de relaxation_mod (pour le calcul de l'epaisseur)            !
!     et adaptation pour le calcul de l'ecoulement de l'eau sous la glace  !
!     selon une equation de Darcy avec un shema amont.                     !
!                                                                          !
! ************************************************************************ !

MODULE RELAXATION_WATER_MOD

CONTAINS


  subroutine RELAXATION_WAT(NXX,NYY,DT,DX,vieuxHWATER,BMELT,INFILTR,PGMX,PGMY,KOND,KONDMAX,HWATER)
    implicit none


    ! declaration des variables en entree
    !------------------------------------------------
    INTEGER, intent(in) :: NXX, NYY       !< defini la taille des tableaux
    REAL,    intent(in) ::  DT            !< pas de temps court
    REAL,    intent(in) ::  DX            !< pas en x
    REAL,    intent(in) :: INFILTR        !< basal infiltration (lose of water) 
    REAL,    intent(in) :: KONDMAX        !< maximum hydaulic conductivity (outside ice sheet)

    REAL,dimension(NXX,NYY), intent(in) :: vieuxHWATER    !< H au pas de temps precedent 'o'
    REAL,dimension(NXX,NYY), intent(in) :: BMELT     !< basal melting  'o'
    REAL,dimension(NXX,NYY), intent(in) :: PGMX     !< hydaulic potential gratient '>' 
    REAL,dimension(NXX,NYY), intent(in) :: PGMY     !< hydaulic potential gratient '^'
    REAL,dimension(NXX,NYY), intent(in) :: KOND     !< hydaulic conductivity 'o'

    ! declaration des variables en sortie
    !-------------------------------------
    REAL,dimension(NXX,NYY), intent(out):: HWATER      !< basal water thickness  'o' (pressure equivalent)


    ! declaration des variables locales
    !----------------------------------
    INTEGER :: I,J
    REAL  :: TESTH 
    REAL,dimension(NXX,NYY) :: ARELAX,BRELAX,CRELAX,DRELAX,ERELAX,FRELAX
    REAL,dimension(NXX,NYY) :: DELTAH
    REAL :: RESTE,DELH,VH
    INTEGER :: ntour
    INTEGER :: mbord
    REAL  :: DTSRGDX 
    LOGICAL :: STOPP
    REAL,dimension(NXX,NYY) :: KMX, KMY
    !    REAL  :: RHO,RHOW,RHOG  !,SECYEAR!! ice density, water density, density*acceleration

    HWATER(:,:)= vieuxHWATER(:,:)

    !   calcul de kmx et kmx a partir de KOND
    !   conductivite hyrdraulique sur les noeuds mineurs
    !   moyenne harmonique
    !   ----------------------------------------

    do j=1,nyy
       do i=2,nxx
          if  (kond(i,j)==0.or.kond(i-1,j)==0) then
             kmx(i,j)=0. ! to avoid division by o
          else
             kmx(i,j)=2*(kond(i,j)*kond(i-1,j))/(kond(i,j)+kond(i-1,j))
          endif

       end do
    end do

    do j=2,nyy
       do i=2,nxx
          if  (kond(i,j)==0.or.kond(i,j-1)==0) then
             kmy(i,j)=0. ! to avoid division by o
          else
             kmy(i,j)=2*(kond(i,j)*kond(i,j-1))/(kond(i,j)+kond(i,j-1))
          endif

       enddo
    enddo


    !   attribution des coefficients  arelax ....
    !   ----------------------------------------
    !    SECYEAR=365.*24.*3600.
    !   rho=910.
    !    rhow=1000.
    !    rhog=rhow*9.81
    !    dtsrgdx=dt/(rhog*DX) a mon avis c'est rhow qu'il fallait utiliser. Maintenant cette 
    !    division est faite dans eaubasale 

    dtsrgdx=dt/DX


    do J=2,NYY-1
       do I=2,NXX-1

          FRELAX(I,J)= VIEUXHWATER(I,J)+(BMELT(I,J)-INFILTR)*DT !

          !          selon x
          !                                     ecoulement vers la droite (sens axe x)
          if ((PGMX(I,J).ge.0.).and.(PGMX(I+1,J).ge.0.)) then
             ARELAX(I,J)=-KMX(I,J)*PGMX(I,J)*dtsrgdx
             BRELAX(I,J)=0.
             CRELAX(I,J)=KMX(I+1,J)*PGMX(I+1,J)*dtsrgdx

             !                                     ecoulement vers la gauche
          else if ((PGMX(I,J).le.0.).and.(PGMX(I+1,J).le.0.)) then
             ARELAX(I,J)=0.
             BRELAX(I,J)=KMX(I+1,J)*PGMX(I+1,J)*dtsrgdx
             CRELAX(I,J)=-KMX(I,J)*PGMX(I,J)*dtsrgdx

             !                                     ecoulement divergent
          else if ((PGMX(I,J).le.0.).and.(PGMX(I+1,J).ge.0.)) then
             ARELAX(I,J)=0.
             BRELAX(I,J)=0.
             CRELAX(I,J)=(KMX(I+1,J)*PGMX(I+1,J)-KMX(I,J)*PGMX(I,J))*dtsrgdx

             !                                     ecoulement convergent
          else if ((PGMX(I,J).ge.0.).and.(PGMX(I+1,J).le.0.)) then
             ARELAX(I,J)=-KMX(I,J)*PGMX(I,J)*dtsrgdx
             BRELAX(I,J)=KMX(I+1,J)*PGMX(I+1,J)*dtsrgdx
             CRELAX(I,J)=0.
          endif
  


          !          selon y 
          !                                 ecoulement vers le haut (sens axe y)
          if ((PGMY(I,J).ge.0.).and.(PGMY(I,J+1).ge.0.)) then
             DRELAX(I,J)=-KMY(I,J)*PGMY(I,J)*dtsrgdx
             ERELAX(I,J)=0.
             CRELAX(I,J)=CRELAX(I,J)+1. +(KMY(I,J+1)*PGMY(I,J+1)*dtsrgdx)

             !                                     ecoulement vers le bas
          else if ((PGMY(I,J).le.0.).and.(PGMY(I,J+1).le.0.)) then
             DRELAX(I,J)=0.
             ERELAX(I,J)=KMY(I,J+1)*PGMY(I,J+1)*dtsrgdx
             CRELAX(I,J)=CRELAX(I,J)+1. -(KMY(I,J)*PGMY(I,J)*dtsrgdx)

             !                                     ecoulement divergent
          else if ((PGMY(I,J).le.0.).and.(PGMY(I,J+1).ge.0.)) then
             DRELAX(I,J)=0.
             ERELAX(I,J)=0.
             CRELAX(I,J)=CRELAX(I,J)+1.+((KMY(I,J+1)*PGMY(I,J+1)-KMY(I,J)*PGMY(I,J))*dtsrgdx)

             !                                     ecoulement convergent
          else if ((PGMY(I,J).ge.0.).and.(PGMY(I,J+1).le.0.)) then
             DRELAX(I,J)=-KMY(I,J)*PGMY(I,J)*dtsrgdx
             ERELAX(I,J)=KMY(I,J+1)*PGMY(I,J+1)*dtsrgdx
             CRELAX(I,J)=CRELAX(I,J)+1.
          endif

       end do
    end do

    ! Boucle de relaxation :
    ! ----------------------
    STOPP = .false.

    ntour=0

    Do 362 WHILE(.NOT.STOPP)
       ntour=ntour+1

       do j=2,NYY-1
          do i=2,NXX-1

             RESTE = (((ARELAX(I,J)*HWATER(I-1,J) + BRELAX(I,J)*HWATER(I+1,J)) &
                  + (DRELAX(I,J)*HWATER(I,J-1) + ERELAX(I,J)*HWATER(I,J+1))) &
                  + CRELAX(I,J)*HWATER(I,J))- FRELAX(I,J)

             DELTAH(I,J) = RESTE/CRELAX(I,J)             

          end do
       end do

      

       ! il faut faire le calcul suivant dans une autre boucle car RESTE est fonction
       ! de hwater sur les points voisins.
       do j=2,NYY-1
          do i=2,NXX-1
             HWATER(I,J) = HWATER(I,J) - DELTAH(I,J)
          end do
       end do

       ! critere d'arret:

      

       Delh=0
       Vh=0

       DO j=2,NYY-1
          DO i=2,NXX-1

             !   write(6,*) I,J,delh,deltah(i,j)
             Delh=Delh+deltah(i,j)**2
             !            Vh=Vh+h(i,j)**2.
          END DO
       END DO

!       write(6,*) delh,maxval(deltah),minval(deltah)
       !      testh=SQRT(Delh/Vh)
       if (delh.gt.0.) then
          testh=SQRT(Delh)/((NXX-2)*(NYY-2))
       else
          testh=0.
       endif
       STOPP = (testh.lt.1.E-3).or.(ntour.gt.1000)


362    Continue

     end subroutine relaxation_wat

end module relaxation_water_mod
